/*
 * xpfdagstest.c: Just runs routines in xpf_interface.c: for debugging only.
 */

#include <stdio.h>
extern int xpfPrintf();
extern int exit();
extern int gettimeofday();
#include <sys/time.h>
#include "xpf.h"
#include "xpf_internal.h"
#include "pattern.h"

#define NUMLOOKUPS	100000
#define NUMSETUPS	10000
#define NUMLAUNCHES	10000	/* must be <= above */
#define CHECK_MOREFRAGS	1

int
main()
{
	XPFObject	pf;
	unsigned char	msg1[4096];
	unsigned char	msg2[952];
	int		len1 = 4096;
	int		len2 = 952;
	struct timeval	begin, end;
	int		i, j, k;
	XPFInformation	info;
	int		cache;
	XPFObject	pfarray[NUMSETUPS];
	XPFPattern	*pat;

	/*
	 * Paper Test Parameters:
	 * 1. How long to look for a TCP/IP patternlist with 1 session without IP program on various machines.
	 * 2. How long to look for a TCP/IP patternlist with 1, 10, ... 100 sessions with and without IP program.
	 * 3. How long to look for dummy TCP/TCP/TCP.../IP patternlist without programs, #TCPs being varied.
	 *
	 * NOTE1: xpf does not call any network-host compatibility functions -- while setting up stuff, you
	 * must set up things that take care of that, i.e., use htons/l rather than ntohs/l.
	 * NOTE2: fields: 16..19=destaddr, 9=protid, 6..7=morefrag, 4..5=msgid, 22..23=destport FOR BOTH TCP/UDP.
	 */
	if (!xpfCreateObject(&pf)) exit (2);
	msg1[16] = 192;
	msg1[17] = 12;
	msg1[18] = 69;
	msg1[19] = 111;
	msg1[9] = 6;	/* TCP ON IP */
	msg1[6] = 0x20;	/* more frags */
	msg1[7] = 0x00;
	msg1[22] = 0x96;
	msg1[23] = 0x00;

	/********************** TEST1 -- SINGLE TCP/IP LOOKUP TIME ON VARIOUS ARCHITECTURES WITH AND WITHOUT CACHE *************************/

#if	CHECK_MOREFRAGS
	cache = 1;
#else	CHECK_MOREFRAGS
	cache = 0;
#endif	CHECK_MOREFRAGS
	setup_tcpip_pattern(pf, htons(0x9600), htonl((192<<24) + (12<<16) + (69 << 8) + 111), cache, pf);
	/* xpfPrintPATTree(((XPFState *)pf)->xpfAssociationSet, 0); */
	msg2[16] = 192;
	msg2[17] = 12;
	msg2[18] = 69;
	msg2[19] = 111;
	msg2[9] = 6;	/* TCP ON IP */
	msg2[6] = 0x00;	/* no more frags */
	msg2[7] = 0x00;
	msg2[22] = 0x96;
	msg2[23] = 0x00;
	*(unsigned short *)&(msg2[4]) = 1;	/* same message id of 1 */
	gettimeofday(&begin, NULL);
	for (i=0; i<NUMLOOKUPS; i++) {
		/* info = xpf_read(pf, msg2, len2); */
		xpfLookupAssociation(pf, msg2, len2, &info);
	}
	gettimeofday(&end, NULL);
	printf("Time for 1 session %d lookups without cache = %d us, info = %d\n",
		NUMLOOKUPS, (end.tv_sec - begin.tv_sec)*1000000 + (end.tv_usec - begin.tv_usec), info);
	/* xpfPrintPATTree(((XPFState *)pf)->xpfAssociationSet, 0); */
	remove_tcpip_pattern(pf, htons(0x9600), htonl((192<<24) + (12<<16) + (69 << 8) + 111), cache);
	xpfDestroyObject(pf);
	if (!xpfCreateObject(&pf)) exit (2);

	cache = 1;
	setup_tcpip_pattern(pf, htons(0x9600), htonl((192<<24) + (12<<16) + (69 << 8) + 111), cache);
	/* xpfPrintPATTree(((XPFState *)pf)->xpfAssociationSet, 0); */
	msg2[16] = 192;
	msg2[17] = 12;
	msg2[18] = 69;
	msg2[19] = 111;
	msg2[9] = 6;	/* TCP ON IP */
	msg2[6] = 0x20;	/* more frags */
	msg2[7] = 0x00;
	msg2[22] = 0x96;
	msg2[23] = 0x00;
	*(unsigned short *)&(msg2[4]) = 1;	/* same message id of 1 */
	gettimeofday(&begin, NULL);
	for (i=0; i<NUMLOOKUPS; i++) {
		/* info = xpf_read(pf, msg2, len2); */
		xpfLookupAssociation(pf, msg2, len2, &info);
	}
	gettimeofday(&end, NULL);
	printf("Time for 1 session %d lookups with cache = %d us, info = %d\n",
		NUMLOOKUPS, (end.tv_sec - begin.tv_sec)*1000000 + (end.tv_usec - begin.tv_usec), info);
	/* xpfPrintPATTree(((XPFState *)pf)->xpfAssociationSet, 0); */
	remove_tcpip_pattern(pf, htons(0x9600), htonl((192<<24) + (12<<16) + (69 << 8) + 111), cache);
	/* xpfPrintPATTree(((XPFState *)pf)->xpfAssociationSet, 0); */
	xpfDestroyObject(pf);
	if (!xpfCreateObject(&pf)) exit (2);

	/********************** TEST2 -- MANY SESSION TCP/IP LOOKUP TIME WITH AND WITHOUT CACHE *************************/

#if	CHECK_MOREFRAGS
	cache = 1;
#else	CHECK_MOREFRAGS
	cache = 0;
#endif	CHECK_MOREFRAGS
	msg2[16] = 192;
	msg2[17] = 12;
	msg2[18] = 69;
	msg2[19] = 111;
	msg2[9] = 6;	/* TCP ON IP */
	msg2[6] = 0x00;	/* no more frags */
	msg2[7] = 0x00;
	msg2[22] = 0;
	msg2[23] = 0;
	for (j=0; j<100; j+=10) {
		for (k=0; k<10; k++) setup_tcpip_pattern(pf, htons(j + k), htonl((192<<24) + (12<<16) + (69 << 8) + 111), cache);
		/* xpfPrintPATTree(((XPFState *)pf)->xpfAssociationSet, 0); */
		*(unsigned short *)&(msg2[4]) = (j/10)%2 + 1;	/* message id alternates between 1 and 2 */
		gettimeofday(&begin, NULL);
		for (i=0; i<NUMLOOKUPS; i++) {
			/* info = xpf_read(pf, msg2, len2); */
			xpfLookupAssociation(pf, msg2, len2, &info);
		}
		gettimeofday(&end, NULL);
		printf("Time for %d sessions %d lookups without cache = %d us, info = %d\n",
			j+k, NUMLOOKUPS, (end.tv_sec - begin.tv_sec)*1000000 + (end.tv_usec - begin.tv_usec), info);
	}
	for(j=0; j<100; j+=10) {
		for (k=0; k<10; k++) remove_tcpip_pattern(pf, htons(j + k), htonl((192<<24) + (12<<16) + (69 << 8) + 111), cache);
	}
	/* xpfPrintPATTree(((XPFState *)pf)->xpfAssociationSet, 0); */
	xpfDestroyObject(pf);
	if (!xpfCreateObject(&pf)) exit (2);

	cache = 1;
	msg2[16] = 192;
	msg2[17] = 12;
	msg2[18] = 69;
	msg2[19] = 111;
	msg2[9] = 6;	/* TCP ON IP */
	msg2[6] = 0x20;	/* more frags */
	msg2[7] = 0x00;
	msg2[22] = 0;
	msg2[23] = 0;
	for (j=0; j<100; j+=10) {
		for (k=0; k<10; k++) setup_tcpip_pattern(pf, htons(j + k), htonl((192<<24) + (12<<16) + (69 << 8) + 111), cache);
		/* xpfPrintPATTree(((XPFState *)pf)->xpfAssociationSet, 0); */
		*(unsigned short *)&(msg2[4]) = (j/10)%2 + 1;	/* message id alternates between 1 and 2 */
		gettimeofday(&begin, NULL);
		for (i=0; i<NUMLOOKUPS; i++) {
			/* info = xpf_read(pf, msg2, len2); */ xpfLookupAssociation(pf, msg2, len2, &info);
		}
		gettimeofday(&end, NULL);
		printf("Time for %d sessions %d lookups with cache = %d us, info = %d\n",
			j+k, NUMLOOKUPS, (end.tv_sec - begin.tv_sec)*1000000 + (end.tv_usec - begin.tv_usec), info);
	}
	for(j=0; j<100; j+=10) {
		for (k=0; k<10; k++) remove_tcpip_pattern(pf, htons(j + k), htonl((192<<24) + (12<<16) + (69 << 8) + 111), cache);
	}
	/* xpfPrintPATTree(((XPFState *)pf)->xpfAssociationSet, 0); */
	xpfDestroyObject(pf);
	if (!xpfCreateObject(&pf)) exit (2);

	/********************** TEST3 -- SINGLE DUMMY TCP/IP LOOKUP TIME WITHOUT CACHE FOR VARIABLE NO. OF TCPS *************************/

	/* Dummy guys check for 0 every where. Otherwise, have to fill up message with proper fields, which is a pain. After all, only #filters matters */
	bzero(msg2, len2);
	msg2[9] = 6;	/* TCP ON IP */
	for (j=0; j<10; j++) {
		if (j == 7) break;
		setup_variable_pattern(pf, htons(0), htonl(0), j);
		/* xpfPrintPATTree(((XPFState *)pf)->xpfAssociationSet, 0); */
		info = NULL;
		gettimeofday(&begin, NULL);
		for (i=0; i<NUMLOOKUPS; i++) {
			/* info = xpf_read(pf, msg2, len2); */
			xpfLookupAssociation(pf, msg2, len2, &info);
		}
		gettimeofday(&end, NULL);
		printf("Time for %d TCP protocols's %d lookups without cache = %d us, info = %d\n",
			j, NUMLOOKUPS, (end.tv_sec - begin.tv_sec)*1000000 + (end.tv_usec - begin.tv_usec), info);
	}
	/*
	-- DON'T BOTHER: ELSE, MUST IMPLEMENT remove_variable_pattern() in pattern.c !!!
	for(j=0; j<10; j+=10) {
		for (k=0; k<10; k++) remove_variable_pattern(pf, htons(0), htonl(0), j);
	}
	*/
	/* xpfPrintPATTree(((XPFState *)pf)->xpfAssociationSet, 0); */

	xpfDestroyObject(pf);
XXXX:
	if (!xpfCreateObject(&pf)) exit (2);

	/****************************** TEST4: Fragmentation overhead (launching a program) *****************************/

	cache = 1;
	setup_tcpip_pattern(pf, htons(0x9600), htonl((192<<24) + (12<<16) + (69 << 8) + 111), cache);
	/* xpfPrintPATTree(((XPFState *)pf)->xpfAssociationSet, 0); */
	msg2[16] = 192;
	msg2[17] = 12;
	msg2[18] = 69;
	msg2[19] = 111;
	msg2[9] = 6;	/* TCP ON IP */
	msg2[6] = 0x20;	/* more frags */
	msg2[7] = 0x00;
	msg2[22] = 0x96;
	msg2[23] = 0x00;
	gettimeofday(&begin, NULL);
	for (i=0; i<NUMLAUNCHES; i++) {
		*(unsigned short *)&(msg2[4]) = i+1;	/* message id keeps changing: as if one fragment of each msgid */
		/* info = xpf_read(pf, msg2, len2); */
		xpfLookupAssociation(pf, msg2, len2, &info);
	}
	gettimeofday(&end, NULL);
	printf("Time for 1 session %d launches = %d us, info = %d\n",
		NUMLAUNCHES, (end.tv_sec - begin.tv_sec)*1000000 + (end.tv_usec - begin.tv_usec), info);
#if	0
	remove_tcpip_pattern(pf, htons(0x9600), htonl((192<<24) + (12<<16) + (69 << 8) + 111), cache);
	/* xpfPrintPATTree(((XPFState *)pf)->xpfAssociationSet, 0); */
#endif	0
	xpfDestroyObject(pf);
	if (!xpfCreateObject(&pf)) exit (2);

	/********************** TEST5 -- SETUP/REMOVE TIMES FOR INITIAL AND NEW TCP/IP PATTERNS **************************/

	for (j=0; j<NUMSETUPS; j++)
		if (!xpfCreateObject(&pfarray[j])) {
			printf("xpfCreateObject failed after %d successes\n", j);
			xpfSetParameters(pfarray[j], XPF_SET_MAXASSOCIATIONS, NUMLOOKUPS*2);
			exit(2);
		}
	if (-1 == create_xip_filter(htons(0x9600), htonl((192<<24) + (12<<16) + (69 << 8) + 111), "tcp", &pat)) exit(2);

	/* SETUP */

	gettimeofday(&begin, NULL);
	for (j=0; j<NUMSETUPS; j++)
		if (!xpfSetupAssociation(pfarray[j], pat, 0x11111111)) {
			printf("Initial setups failed after %d successes\n", j);
			break;
		}
	gettimeofday(&end, NULL);
	printf("Time to setup %d initial TCP/IP patterns = %d us\n",
		j, (end.tv_sec - begin.tv_sec)*1000000 + (end.tv_usec - begin.tv_usec));

	/* REMOVE */

	gettimeofday(&begin, NULL);
	for (j=0; j<NUMSETUPS; j++)
		if (!xpfRemoveAssociation(pfarray[j], pat)) {
			printf("Initial removes failed after %d successes\n", j);
			break;
		}
	gettimeofday(&end, NULL);
	printf("Time to remove %d initial TCP/IP patterns = %d us\n",
		j, (end.tv_sec - begin.tv_sec)*1000000 + (end.tv_usec - begin.tv_usec));

	/* SETUP */

	if (!xpfSetupAssociation(pfarray[0], pat, 0x11111111)) exit(2);
	gettimeofday(&begin, NULL);
	for (j=0; j<NUMLOOKUPS; j++)
		if (!xpfSetupAssociation(pfarray[0], pat, 0x11111111)) {
			printf("New setups failed after %d successes\n", j);
			break;
		}
	gettimeofday(&end, NULL);
	printf("Time to setup %d new TCP/IP patterns = %d us\n",
		j, (end.tv_sec - begin.tv_sec)*1000000 + (end.tv_usec - begin.tv_usec));

	/* REMOVE */

	gettimeofday(&begin, NULL);
	for (j=0; j<NUMLOOKUPS; j++)
		if (!xpfRemoveAssociation(pfarray[0], pat)) {
			printf("New removes failed after %d successes\n", j);
			break;
		}
	gettimeofday(&end, NULL);
	printf("Time to remove %d new TCP/IP patterns = %d us\n",
		j, (end.tv_sec - begin.tv_sec)*1000000 + (end.tv_usec - begin.tv_usec));

}
